local name, ns = ...
WhoRoll = LibStub("AceAddon-3.0"):NewAddon(name, "AceEvent-3.0")
local WhoRoll = WhoRoll

local deformat = LibStub("LibDeformat-3.0")

local debug = ns.debug
WhoRoll.debug = debug

local rollHistory = {}
WhoRoll.rollHistory = rollHistory

local function initWithNewRoll(self, key)
	local value = rawget(self, key)
	if not value then
		value = {}
		value.rolls = {}
		value.item = key
		rawset(self, key, value)
	end
	return value
end

local currentLoots = setmetatable({}, { __index = initWithNewRoll, })
WhoRoll.currentLoots = currentLoots

-- globals
local _G = _G
local GREED = _G.GREED
local NEED = _G.NEED
local PASS = _G.PASS
local DISENCHANT = "DISENCHANT"
local YOU = _G.YOU
local GetItemInfo = _G.GetItemInfo
local GetLootRollItemLink = _G.GetLootRollItemLink
local GetLootRollItemInfo = _G.GetLootRollItemInfo

--[===[@debug@
function WhoRoll:InsertTestLoots()
	local _, testItem = GetItemInfo(32483)
	local testLoot = {
		item = testItem,
		winner = UnitName("player"),
		choice = GREED,
	}
	table.insert(rollHistory, testLoot)
	local _, testItem = GetItemInfo(43408)
	local testRoll = {
		item = testItem,
		rolls = {
			[UnitName("player")] = NEED,
			test1 = GREED,
			test2 = DISENCHANT,
		}
	}
	currentLoots[testItem] = testRoll
	self:DebugBroker()
end
--@end-debug@]===]

local function debug(msg)
	--[===[@debug@
	ChatFrame4:AddMessage(msg)
	--@end-debug@]===]
end

function WhoRoll:OnEnable()
	self:RegisterEvent("START_LOOT_ROLL")
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:UpdateTooltip()
end

function WhoRoll:START_LOOT_ROLL(_, rollId, rollTime)
	local link = GetLootRollItemLink(rollId)
	debug("START_LOOT_ROLL: .. " .. rollId .. "... " .. link)
	initWithNewRoll(currentLoots, link)
	self:UpdateTooltip()
end

local playerName = UnitName("player")
local parsers = {
	[LOOT_ROLL_GREED_SELF] = function(message, pattern)
			return playerName, deformat(message, pattern), GREED
		end,
	[LOOT_ROLL_GREED] = function(message, pattern)
			local name, item = deformat(message, pattern)
			return name, item, GREED
		end,
	[LOOT_ROLL_NEED_SELF] = function(message, pattern)
			return playerName, deformat(message, pattern), NEED
		end,
	[LOOT_ROLL_NEED] = function(message, pattern)
			local name, item = deformat(message, pattern)
			return name, item, NEED
		end,
	[LOOT_ROLL_DISENCHANT_SELF] = function(message, pattern)
			return playerName, deformat(message, pattern), DISENCHANT
		end,
	[LOOT_ROLL_DISENCHANT] = function(message, pattern)
			local name, item = deformat(message, pattern)
			return name, item, DISENCHANT
		end,
	[LOOT_ROLL_PASSED_SELF] = function(message, pattern)
			return playerName, deformat(message, pattern), PASS
		end,
	[LOOT_ROLL_PASSED] = function(message, pattern)
			local name, item = deformat(message, pattern)
			return name, item, PASS
		end,
}

parsers[LOOT_ROLL_PASSED_SELF_AUTO] = parsers[LOOT_ROLL_PASSED_SELF]
parsers[LOOT_ROLL_PASSED_AUTO] = parsers[LOOT_ROLL_PASSED]
parsers[LOOT_ROLL_PASSED_AUTO_FEMALE] = parsers[LOOT_ROLL_PASSED]

local matchers = {}
for k in pairs(parsers) do
	matchers[k] = k:gsub("%%s", ".+")
end
_G.matchers = matchers

local matcher_LOOT_ROLL_WON = LOOT_ROLL_WON:gsub("%%s", ".+")
local matcher_LOOT_ROLL_ALL_PASSED = LOOT_ROLL_ALL_PASSED:gsub("%%s", ".+")

local function trackRoll(name, itemLink, selection)
	debug("track Roll: " .. (name or "nil") .. " selected on " .. (itemLink or "nil"))
	local thisLoot = currentLoots[itemLink]
	thisLoot.rolls[name] = selection
end

local function setWinner(itemLink, winner)
	local lootRoll = currentLoots[itemLink]
	lootRoll.winner = winner
	lootRoll.choice = lootRoll.rolls[winner]
	wipe(lootRoll.rolls)
	lootRoll.rolls = nil
	table.insert(rollHistory, value)
end

local function removeRoll(itemLink)
	if not rawget(currentLoots, itemLink) then return end
	wipe(currentLoots[itemLink])
	currentLoots[itemLink] = nil
end

function WhoRoll:CHAT_MSG_LOOT(_, message)
	local itemLink, name, selection

	for k,v in pairs(matchers) do
		if message:match(v) then
			local name, itemLink, selection = parsers[k](message, k)
			trackRoll(name, itemLink, selection)
			self:UpdateTooltip()
			return
		end
	end

	if deformat(message,LOOT_ROLL_WON) then
		name, itemLink = deformat(message,LOOT_ROLL_WON)
		if name == YOU then
			name = playerName
		end
		setWinner(itemLink, name)
	elseif deformat(message, LOOT_ROLL_ALL_PASSED) then
		itemLink = deformat(message,LOOT_ROLL_ALL_PASSED)
		removeRoll(itemLink)
	else
		--didn't match anything
		debug("string could not be matched")
		debug(message)
		return
	end
	self:UpdateTooltip();
end
